@@ -13,7 +13,7 @@ import warnings
 import uuid
 
 from wtforms.fields import BooleanField, DecimalField, DateField, \
-    DateTimeField, FieldList, FloatField, FileField, FormField, \
+    DateTimeField, FieldList, FloatField, FormField, \
     HiddenField, IntegerField, PasswordField, RadioField, SelectField, \
     SelectMultipleField, SubmitField, TextField, TextAreaField
 
@@ -26,6 +26,8 @@ from wtforms.widgets import CheckboxInput, FileInput, HiddenInput, \
     ListWidget, PasswordInput, RadioInput, Select, SubmitInput, \
     TableWidget, TextArea, TextInput
 
+from wtforms.fields import FileField as _FileField
+
 try:
     import sqlalchemy
     _is_sqlalchemy = True
@@ -77,6 +79,17 @@ def _generate_csrf_token():
     return str(uuid.uuid4())
 
 
+class FileField(_FileField):
+
+    @property
+    def file(self):
+        """
+        Returns FileStorage class if available from request.files
+        or None
+        """
+        return request.files.get(self.name, None)
+
+
 class Form(BaseForm):
 
     """
@@ -129,17 +142,11 @@ class Form(BaseForm):
             # ensure csrf validation occurs ONLY when formdata is passed
             # in case "csrf" is the only field in the form
 
-            if not formdata:
+            if not formdata and not request.files:
                 self.csrf_is_valid = False
             else:
                 self.csrf_is_valid = None
 
-            if request.files:
-
-                for name, field in self._fields.iteritems():
-                    if isinstance(field, FileField) and name in request.files:
-                        field.file = request.files[name]
-
         super(Form, self).process(formdata, obj, **kwargs)
 
     @property
@@ -190,17 +197,20 @@ class Form(BaseForm):
         Wraps hidden fields in a hidden DIV tag, in order to keep XHTML 
         compliance.
 
-        :versionadded: 0.3
+        .. versionadded:: 0.3
 
         :param fields: list of hidden field names. If not provided will render
                        all hidden fields, including the CSRF field.
         """
 
         if not fields:
-            fields = [f.name for f in self if isinstance(f, HiddenField)]
+            fields = [f for f in self if isinstance(f, HiddenField)]
 
         rv = [u'<div style="display:none;">']
-        rv += [unicode(getattr(self, field)) for field in fields]
+        for field in fields:
+            if isinstance(field, basestring):
+                field = getattr(self, field)
+            rv.append(unicode(field))
         rv.append(u"</div>")
 
         return Markup(u"".join(rv))
