@@ -2,10 +2,10 @@ from __future__ import with_statement
 
 import re
 
-from flask import Flask, render_template, jsonify
+from flask import Flask, Response, render_template, jsonify
 from flaskext.testing import TestCase as _TestCase
 from flaskext.wtf import Form, TextField, FileField, HiddenField, \
-    SubmitField, Required
+    SubmitField, Required, FieldList
 
 class TestCase(_TestCase):
     
@@ -18,9 +18,17 @@ class TestCase(_TestCase):
         class HiddenFieldsForm(Form):
             name = HiddenField()
             url = HiddenField()
+            method = HiddenField()
             secret = HiddenField()
             submit = SubmitField("Submit")
 
+            def __init__(self, *args, **kwargs):
+                super(HiddenFieldsForm, self).__init__(*args, **kwargs)
+                self.method.name = '_method'
+
+        class SimpleForm(Form):
+            pass
+
         app = Flask(__name__)
         app.secret_key = "secret"
         
@@ -37,7 +45,15 @@ class TestCase(_TestCase):
                                    form=form,
                                    name=name)
 
-            
+        @app.route("/simple/", methods=("POST",))
+        def simple():
+            form = SimpleForm()
+            form.validate()
+            assert form.csrf_enabled
+            assert not form.validate()
+            assert not form.validate()
+            return Response("OK")
+
         @app.route("/hidden/")
         def hidden():
 
@@ -67,9 +83,24 @@ class TestFileUpload(TestCase):
 
             upload = FileField("Upload file")
 
+        class MultipleFileUploadForm(Form):
+
+            uploads = FieldList(FileField("upload"), min_entries=3)
+
+
         app = super(TestFileUpload, self).create_app()
         app.config['CSRF_ENABLED'] = False
 
+        @app.route("/upload-multiple/", methods=("POST",))
+        def upload_multiple():
+            form = MultipleFileUploadForm()
+            if form.validate_on_submit():
+                for upload in form.uploads.entries:
+                    assert upload.file is not None
+
+            return Response("OK")
+
+
         @app.route("/upload/", methods=("POST",))
         def upload():
             form = FileUploadForm()
@@ -87,6 +118,14 @@ class TestFileUpload(TestCase):
         
         return app
 
+
+    def test_multiple_files(self):
+
+        fps = [self.app.open_resource("flask.png") for i in xrange(3)]
+        data = [("uploads-%d" % i, fp) for i, fp in enumerate(fps)] 
+        response = self.client.post("/upload-multiple/", data=dict(data))
+        assert response.status_code == 200
+
     def test_valid_file(self):
         
         with self.app.open_resource("flask.png") as fp:
@@ -132,7 +171,9 @@ class TestHiddenTag(TestCase):
     def test_hidden_tag(self):
 
         response = self.client.get("/hidden/")
-        assert response.data.count('type="hidden"') == 4
+        assert response.data.count('type="hidden"') == 5
+        assert 'name="_method"' in response.data
+
 
 class TestCSRF(TestCase):
 
@@ -154,6 +195,11 @@ class TestCSRF(TestCase):
         response = self.client.post("/", data={"name" : "danny"})
         assert 'DANNY' in response.data
 
+    def test_validate_twice(self):
+
+        response = self.client.post("/simple/", data={})
+        self.assert_200(response)
+
     def test_ajax(self):
 
         response = self.client.post("/ajax/", 
